var update_interval = 20; //update interval (sec) of refreshing actual light, pump and ozone state
var error_resend_timeout = 2000; // timeout (ms) for resending request if previous request failed 

/*    Change status of output on webpage  */
function set_status(controller, new_status) {
    if (new_status == 0) {
        $("#" + controller + "_control li[name='off']").addClass('active').siblings().removeClass('active');
        console.log("changing " + controller + " to off");
    } else if (new_status == 1) {
        $("#" + controller + "_control li[name='on']").addClass('active').siblings().removeClass('active');
        console.log("changing " + controller + " to on");
    } else if (new_status == 2) {
        $("#" + controller + "_control li[name='auto']").addClass('active').siblings().removeClass('active');
        console.log("changing " + controller + " to auto");
    }
}

/*    Change state of output   */
function set_state(controller, new_state) {
    if (new_state == 0) {
        $("#" + controller + "_control span[name='state']").addClass('label-danger').removeClass('label-default label-success');
        console.log("changing " + controller + " state to stop");
    } else if (new_state == 1) {
        $("#" + controller + "_control span[name='state']").addClass('label-success').removeClass('label-default label-danger');
        console.log("changing " + controller + " state to play");
    }
}

/*    Update actual time on website   */
function update_time(tmv) {
    var d = new Date();
    // ds1307 uses year in range 0 - 99
    d.setFullYear(tmv.year + 2000);
    // ds1307 uses month in range 1 - 12
    d.setMonth(tmv.month - 1);
    d.setDate(tmv.date);
    d.setHours(tmv.hour);
    d.setMinutes(tmv.min);
    d.setSeconds(tmv.sec);
    $("#last_update_time").html("Last update: " + d.toLocaleString());
}

/*    Send post with actual time to server    */
function set_clock_ajax(data_string) {
    $.ajax({
        url: "/set_clock",
        type: "POST",
        data: data_string,
        contentType: "text/plain",
        dataType: "json",
        success: function(result) {
            update_time(result.time);
			$("#sel_year").val(tmv.year);
			$("#sel_month").val(tmv.month);
			$("#sel_date").val(tmv.date);
			$("#sel_day").val(tmv.day);
			$("#sel_hour").val(tmv.hour);
			$("#sel_min").val(tmv.min);
			$("#sel_sec").val(tmv.sec);
        },
        error: function (request, status, error) {
            console.log(request);
            console.log(status);
            console.log(error);
            setTimeout(function() {set_clock_ajax(data_string);}, error_resend_timeout);
        },
    });
}

/*    Parse time to string    */
function set_clock() {
    var year = $("#sel_year").val().toString();
    var month = $("#sel_month").val().toString();
    var date = $("#sel_date").val().toString();
    var day = $("#sel_day").val().toString();
    var hour = $("#sel_hour").val().toString();
    var min = $("#sel_min").val().toString();
    var sec = $("#sel_sec").val().toString();
    var data_string = year + " " + month + " " + date + " " + day + " " + hour + " " + min + " " + sec;
    set_clock_ajax(data_string);
}

/*    Set status, state and update time via AJAX    */
function trigger_ajax(controller, action) {
    $.ajax({
        url: "/" + controller + "_" + action,
        type: "POST",
        dataType: "json",
        success: function(result) {
            set_status(controller, result.status);
            set_state(controller, result.state);
            update_time(result.time);
        },
        error: function (request, status, error) {
            console.log(request);
            console.log(status);
            console.log(error);
            setTimeout(function() {trigger_ajax(controller, action);}, error_resend_timeout);
        },
    });
}

/*     Trigger action    */
function trigger(controller, action) {
    console.log(controller + "_" + action);
    trigger_ajax(controller, action);
}

/*     Update status of all controlled devices    */
function update_all() {
    trigger("light", "status");
    trigger("pump", "status");
    trigger("ozone", "status");
}

/*     Maximal number of intervals in table   */
var max_intervals = {
    "pump": 10,
    "ozone": 10,
};
var intervals_count = {
    "pump": 0,
    "ozone": 0,
};

/*    Function for adding new interval    */
function add_interval(ctrl) {
    console.log("adding " + ctrl + " interval");
    var hour_spinbox = '<input type="number" min="0" max="23" value="0" style="max-width: 50px;" required>';
    var min_spinbox = '<input type="number" min="0" max="59" value="0" style="max-width: 50px;" required>';

    if (intervals_count[ctrl] < max_intervals[ctrl]) { //max input box allowed
        $("#" + ctrl + "_intervals tbody").append(
                '<tr id="' + ctrl + '_interval' + intervals_count[ctrl] + '"><th name="start_hour">' + hour_spinbox + '</th><th name="start_minute">' + min_spinbox +
                '</th><th name="end_hour">' + hour_spinbox + '</th><th name="end_minute">' + min_spinbox +
                '</th><th><button type="button" class="btn btn-danger"><span class="glyphicon glyphicon-minus"></span></button></th></tr>');
        intervals_count[ctrl]++;
        var tr_to_remove = $("#" + ctrl + "_intervals tbody tr:last");
        $(tr_to_remove).find("button").click(function () {
            console.log("removing " + ctrl + " interval");
            $(tr_to_remove).remove();
            intervals_count[ctrl]--;
        });
    }
    return tr_to_remove;
}

/*    Set parameters of interval - start hour+min and end hour+min    */
function set_interval(interval, values) {
    $(interval).find("[name=start_hour] input").val(values["start_hour"]);
    $(interval).find("[name=start_minute] input").val(values["start_minute"]);
    $(interval).find("[name=end_hour] input").val(values["end_hour"]);
    $(interval).find("[name=end_minute] input").val(values["end_minute"]);
}

/*    Set all intervals    */
function set_intervals(ctrl, intervals, tmv) {
    for (var i = 0; i < intervals.length; i++) {
        var interval = add_interval(ctrl);
        set_interval(interval, intervals[i]);
    }
    update_time(tmv);
}

/*    Delete interval    */
function clear_intervals(ctrl) {
    $("#" + ctrl + "_intervals tbody").empty();
    intervals_count[ctrl] = 0;
}

/*     Post interval parameters to server via AJAX    */
function update_intervals_ajax(ctrl, act, data_values) {
    $.ajax({
        url: "/" + ctrl + "_" + act + "_intervals",
        type: "POST",
        data: data_values,
        contentType: "text/plain",
        dataType: "json",
        success: function(result) {
            console.log("set new values for " + ctrl + " intervals");
            clear_intervals(ctrl);
            set_intervals(ctrl, result["intervals"], result["time"]);
            update_all();
        },
        error: function (request, status, error) {
            console.log(request);
            console.log(status);
            console.log(error);
            setTimeout(function() {update_intervals_ajax(ctrl, act, data_values);}, error_resend_timeout);
        },
    });
}

/*    Update intervals on webpage via data from webserver    */
function update_intervals(ctrl, act) {
    console.log("update " + ctrl + " intervals");
    var data_values = "";
    $("#" + ctrl + "_intervals tbody tr").each(function(index) {
        var start_hour = $(this).find("[name=start_hour] input").val().toString();
        var start_minute = $(this).find("[name=start_minute] input").val().toString();
        var end_hour = $(this).find("[name=end_hour] input").val().toString();
        var end_minute = $(this).find("[name=end_minute] input").val().toString();
        data_values += start_hour + " ";
        data_values += start_minute + " ";
        data_values += end_hour + " ";
        data_values += end_minute;
        data_values += "\n";
    });

    console.log(data_values);

    update_intervals_ajax(ctrl, act, data_values);
}

update_all();
setInterval(update_all, update_interval * 1000); 
update_intervals("pump", "get");
update_intervals("ozone", "get");
